﻿abstract record Person {
	public string? FirstName { get; set; }
	public string? LastName { get; set; }
}

record Customer : Person {
	public int TotalOrders { get; set; }
}

record Employee : Person {
	public decimal Salary { get; set; }
	public DateOnly DateHired { get; set; }
}

record Helper : Person {
	public bool Qualified { get; set; }
}

class Program {
	static void Main(string[] args) {

		Person p1;

		p1 = new Employee { FirstName = "Jiri", LastName = "Vesely", Salary = 1000, DateHired = DateOnly.FromDateTime(DateTime.Today) };
		// p1 = new Employee { FirstName = null, LastName = null, Salary = 1000, DateHired = DateOnly.FromDateTime(DateTime.Today) };

		if (p1 != null) {
			Console.WriteLine(p1);
		}

		if (p1 is Employee e) {
			Console.WriteLine(e.Salary);
		}

		if (!(p1 is Customer)) {
			Console.WriteLine("Not a customer.");
		}

		// C# 9.0

		if (p1 is not Customer) {
			Console.WriteLine("Not a customer.");
		}

		if (p1 is { FirstName: "Pavel", LastName: "Jezek" }) {
			Console.WriteLine("A: {0}", p1);
		}

		if (p1 is not { FirstName: "Pavel", LastName: "Jezek" }) {
			Console.WriteLine("B: {0}", p1);
		}

		Person? p3 = new Helper { FirstName = "John", LastName = "Doe", Qualified = true };
		if (p3 is { } p4) {
			Console.WriteLine("C if: {0}", p4);
		} else {
			Console.WriteLine("C else");
		}
		p3 = null;
		if (p3 is { } p5) {
			Console.WriteLine("D if: {0}", p5);
		} else {
			Console.WriteLine("D else");
		}

		//

		if (p1 is Employee { Salary: > 500, LastName: "Jezek" } e2) {
			Console.WriteLine("E: {0}", e2);
		}

		if (p1 is Employee { Salary: > 500 and <= 2000, LastName: "Jezek" } e3) {
			Console.WriteLine("F: {0}", e3);
		}

		if (p1 is Employee e2x && e2x.Salary > 500 && e2x.Salary <= 2000 && e2x.LastName == "Jezek") {
			Console.WriteLine("F alt: {0}", e2x);
		}

		if (p1 is Employee { Salary: (> 500 and <= 2000) or > 3000, LastName: "Jezek" }) {
			Console.WriteLine("G: {0}", p1);
		}

		if (p1 is Employee { Salary: (> 500 and <= 2000) or not <= 3000, LastName: not "Jezek" }) {
			Console.WriteLine("H: {0}", p1);
		}

		var x = ((Employee) p1).LastName?.Length;

		if (p1 is Employee { Salary: (> 500 and <= 2000) or not <= 3000 } e4 && e4.LastName?.Length > 2) {
			Console.WriteLine("I: {0}", e4);
		}

		if (p1 is Employee { Salary: (> 500 and <= 2000) or not <= 3000 } e5 && e5.LastName is { Length: > 2 }) {
			Console.WriteLine("J: {0}", e5);
		}

		{
			if (p1 is Employee { Salary: (> 500 and <= 2000) or not <= 3000, FirstName: var firstName } e6 && e6.LastName is { Length: > 2, Length: < 10 }) {
				Console.WriteLine("K: {0}", e6);
				Console.WriteLine("  firstName == {0}", firstName);
			}
		}

		{
			if (p1 is Employee { Salary: (> 500 and <= 2000) or not <= 3000, FirstName: var firstName } e7 && e7.LastName is { Length: > 2, Length: var length }) {
				Console.WriteLine("L: {0}", e7);
				Console.WriteLine("  firstName == {0}, length == {1}", firstName, length);
			}
		}
	}
}
