﻿
record Person(string FirstName, string LastName);

record Incident(string Info, Person? Person);

class Program {
	static void Main() {
		var incidents = new List<Incident> {
			new Incident("Head injury from falling apple", new Person("Isaac", "Newton")),
			new Incident("Broken toilet seat on 2nd floor", null)
		};
		
		foreach (var indident in incidents) {
			Console.Write("Indident \"" + indident.Info + "\"");
			if (indident.Person is Person person) {
				Console.WriteLine($" with {person.FirstName} {person.LastName} as affected person!");
			} else {
				// Warning: Dereference of a possibly null reference. 'Person' may be null here
				// Runtime error: Unhandled exception. System.NullReferenceException: Object reference not set to an instance of an object.
				// Console.WriteLine($" with {indident.Person.FirstName} {indident.Person.LastName} as affected person!");
				Console.WriteLine(" ...");
			}
		}

        Console.WriteLine();

		foreach (var indident in incidents) {
			Console.Write("Indident \"" + indident.Info + "\"");
			if (indident.Person is not null) {
				Console.WriteLine($" with {indident.Person.FirstName} {indident.Person.LastName} as affected person!");
			} else {
				// Warning: Dereference of a possibly null reference. 'Person' may be null here
				// Runtime error: Unhandled exception. System.NullReferenceException: Object reference not set to an instance of an object.
				// Console.WriteLine($" with {indident.Person.FirstName} {indident.Person.LastName} as affected person!");
				Console.WriteLine(" ...");
			}
		}

        Console.WriteLine();

		int? l1 = incidents[0].Person?.LastName.Length;
        Console.WriteLine($"l1 == {l1}");
        int? l2 = incidents[1].Person?.LastName.Length;
        Console.WriteLine($"l2 == {l2}");
    }
}